@@ -24,8 +24,9 @@
 #' @param formula INLA formula.  See vignette for example of using customized formula.
 #' @param year_label string vector of year names
 #' @param type.st type for space-time interaction
-#' @param survey.effect logical indicator whether to include a survey iid random effect. If this is set to TRUE, there needs to be a column named 'survey' in the input data frame. In prediction, this random effect term will be set to 0. 
+#' @param survey.effect logical indicator whether to include a survey iid random effect. If this is set to TRUE, there needs to be a column named 'survey' in the input data frame. In prediction, this random effect term will be set to 0.
 #' @param strata.time.effect logical indicator whether to include strata specific temporal trends.  
+#' @param common.trend logical indicator whether all age groups and/or strata share the same linear trend in the temporal main effect. Only used when the temporal main effect is an AR(1) process.
 #' @param hyper which hyperpriors to use. Default to be using the PC prior ("pc"). 
 #' @param pc.u hyperparameter U for the PC prior on precisions.
 #' @param pc.alpha hyperparameter alpha for the PC prior on precisions.
@@ -88,7 +89,7 @@
 #' 
 #' 
 
-smoothCluster <- function(data, family = c("betabinomial", "binomial")[1], age.groups = c("0", "1-11", "12-23", "24-35", "36-47", "48-59"), age.n = c(1,11,12,12,12,12), age.rw.group = c(1,2,3,3,3,3), time.model = c("rw1", "rw2", "ar1")[2], st.time.model = NULL, Amat, bias.adj = NULL, bias.adj.by = NULL, formula = NULL, year_label, type.st = 4, survey.effect = FALSE, strata.time.effect = FALSE, hyper = c("pc", "gamma")[1], pc.u = 1, pc.alpha = 0.01, pc.u.phi = 0.5, pc.alpha.phi = 2/3, pc.u.cor = 0.7, pc.alpha.cor = 0.9,  pc.st.u = NA, pc.st.alpha = NA, pc.st.slope.u = NA, pc.st.slope.alpha = NA, overdisp.mean = 0, overdisp.prec = 0.4, options = list(config = TRUE), control.inla = list(strategy = "adaptive", int.strategy = "auto"), verbose = FALSE, geo = NULL, rw = NULL, ar = NULL, st.rw = NULL, ...){
+smoothCluster <- function(data, family = c("betabinomial", "binomial")[1], age.groups = c("0", "1-11", "12-23", "24-35", "36-47", "48-59"), age.n = c(1,11,12,12,12,12), age.rw.group = c(1,2,3,3,3,3), time.model = c("rw1", "rw2", "ar1")[2], st.time.model = NULL, Amat, bias.adj = NULL, bias.adj.by = NULL, formula = NULL, year_label, type.st = 4, survey.effect = FALSE, common.trend = FALSE, strata.time.effect = FALSE, hyper = c("pc", "gamma")[1], pc.u = 1, pc.alpha = 0.01, pc.u.phi = 0.5, pc.alpha.phi = 2/3, pc.u.cor = 0.7, pc.alpha.cor = 0.9,  pc.st.u = NA, pc.st.alpha = NA, pc.st.slope.u = NA, pc.st.slope.alpha = NA, overdisp.mean = 0, overdisp.prec = 0.4, options = list(config = TRUE), control.inla = list(strategy = "adaptive", int.strategy = "auto"), verbose = FALSE, geo = NULL, rw = NULL, ar = NULL, st.rw = NULL, ...){
 
   # if(family == "betabinomialna") stop("family = betabinomialna is still experimental.")
   # check region names in Amat is consistent
@@ -199,12 +200,11 @@ smoothCluster <- function(data, family = c("betabinomial", "binomial")[1], age.g
     is.spatial <- FALSE
   }
 
-  if(is.spatial){
-
-  } 
-  message("\n  Spatial effect:             bym2", 
-          "\n  Interaction temporal model: ", st.time.model, 
-          "\n  Interaction type:           ", type.st, appendLF=FALSE)
+  if(is.spatial){ 
+    message("\n  Spatial effect:             bym2", 
+            "\n  Interaction temporal model: ", st.time.model, 
+            "\n  Interaction type:           ", type.st, appendLF=FALSE)
+  }
   if(!is.na(pc.st.slope.u) && !is.na(pc.st.slope.alpha)){
     message("\n  Interaction random slopes:  yes", appendLF=FALSE)
   }else{
@@ -410,6 +410,10 @@ smoothCluster <- function(data, family = c("betabinomial", "binomial")[1], age.g
     
     # -- creating IDs for the temporal REs -- #
     dat$time.unstruct <- dat$time.struct <- dat$time.int <- years[match(dat$years, years[, 1]), 2]
+    if(sum(is.na(dat$time.unstruct)) > 0){
+      dat <- dat[!is.na(dat$time.unstruct), ]
+      message("Data contains time periods not in the specified range. These observations are removed.")
+    }
   
       
     x <- expand.grid(1:N, 1:region_count)
@@ -497,11 +501,15 @@ smoothCluster <- function(data, family = c("betabinomial", "binomial")[1], age.g
            formula <- Y ~  f(time.struct, model="ar", order=ar, hyper = hyperar1, constr = TRUE, extraconstr = NULL, values = 1:N) 
 
           }
+         
           # AR1 add slope
-          if(is.main.ar){
-            tmp <- paste(slope.fixed.names, collapse = " + ")
-            formula <- as.formula(paste(c(formula, tmp), collapse = "+"))
-          }
+           if(is.main.ar && !common.trend){
+              tmp <- paste(slope.fixed.names, collapse = " + ")
+              formula <- as.formula(paste(c(formula, tmp), collapse = "+"))
+           }
+           if(is.main.ar && common.trend){
+              formula <- as.formula(paste(c(formula, "time.slope"), collapse = "+"))
+           }
           # Add IID
           formula <- update(formula, ~. + 
                   f(time.unstruct,model="iid", hyper = hyperpc1, values = 1:N))
@@ -532,10 +540,13 @@ smoothCluster <- function(data, family = c("betabinomial", "binomial")[1], age.g
              }
 
             # AR1 add slope
-             if(is.main.ar){
+             if(is.main.ar && !common.trend){
                 tmp <- paste(slope.fixed.names, collapse = " + ")
                 formula <- as.formula(paste(c(formula, tmp), collapse = "+"))
              }
+             if(is.main.ar && common.trend){
+                formula <- as.formula(paste(c(formula, "time.slope"), collapse = "+"))
+             }
 
 
              formula <- update(formula, ~. + 
